@@ -1,10 +1,15 @@
+"""
+Settings specific to Country module
+"""
+
 FLAG_SOURCE = 'https://raw.githubusercontent.com/cristiroma/countries/master/data/flags/SVG/{alpha_2}.svg?sanitize=true'
 
 # put in global settings.py to override
 GEOCODING_SERVICE = 'pelias'
 
 # put in global settings.py to override
-GEOCODER_PELIAS_URL = None
+GEOCODER_PELIAS_URL = 'https://api.geocode.earth/v1'
+GEOCODER_PELIAS_KEY = ''
 GEOCODER_GOOGLE_KEY = ''
 
 GEOCODING_SERVICE_SETTINGS = {
